using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using ReflectionIT;

namespace ReflectionIT.Windows.Forms
{
	#if NETCFDESIGNTIME
	[Designer(typeof(TimeBoxDesigner))]
	[DefaultEvent("TimeChanged")]
	#endif
	public class TimeBox : System.Windows.Forms.Control
	{

		#if NETCFDESIGNTIME
		[Category("Action")]
		#endif
		public event EventHandler TimeChanged;

		private SpinButton _SpinButtons;
		private ReflectionIT.Time _time = new ReflectionIT.Time(0);
		private ReflectionIT.Time _minTime = new ReflectionIT.Time(0);
		private ReflectionIT.Time _maxTime = new ReflectionIT.Time(23, 59);
		private bool _hourSelected = false;
		private bool _minutesSelected = true;
		private int _minuteIncrement = 1;
		private int _hourIncrement = 1;
		private bool _rotate = true;

		public TimeBox() {
			// initialization after the InitForm call
#if NETCFDESIGNTIME
				this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
				this.SetStyle(ControlStyles.DoubleBuffer, true);
				this.SetStyle(ControlStyles.UserPaint, true);
				//this.SetStyle(ControlStyles.Selectable, false);
#endif

			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
		}


		#if NETCFDESIGNTIME
		[
		Category("Appearance"),
		Description("The current time of the timebox."),
		]
		#endif
		public ReflectionIT.Time Time {
			get {
				return (this._time);
			}
			set {
				bool changed = (_time != value);
				this._time = value;

				if (changed) {
					Invalidate();
					OnTimeChanged();
				}
			}
		}

		#if NETCFDESIGNTIME
		public void ResetTime() {
			this.Time = new Time(0);
		}

		public bool ShouldSerializeTime() {
			return this.Time != new Time(0);
		}
		#endif

#if NETCFDESIGNTIME
		[
		Category("Data"),
		Description("Indicates the minimum time for the timebox."),
		]
#endif
		public ReflectionIT.Time Minimum {
			get {
				return (this._minTime);
			}
			set {
				_minTime = value;
			}

		}
#if NETCFDESIGNTIME
		public void ResetMinimum() {
			this.Minimum = new ReflectionIT.Time(0);
		}

		public bool ShouldSerializeMinimum() {
			return this.Minimum != new ReflectionIT.Time(0);
		}
#endif

#if NETCFDESIGNTIME
		[
		Category("Data"),
		Description("Indicates the maximum time for the timebox."),
		]
#endif
		public ReflectionIT.Time Maximum {
			get {
				return (this._maxTime);
			}
			set {
				_maxTime = value;
			}
		}

#if NETCFDESIGNTIME
		public void ResetMaximum() {
			this.Maximum = new ReflectionIT.Time(23, 59);
		}

		public bool ShouldSerializeMaximum() {
			return this.Maximum != new ReflectionIT.Time(23, 59);
		}
#endif

#if NETCFDESIGNTIME
		[
		Category("Data"),
		Description("Indicates the hour amount to increment/decrement on each button click."),
		DefaultValue(1)
		]
#endif		
		public int HourIncrement {
			get {
				return (this._hourIncrement);
			}
			set {
				this._hourIncrement = value;
			}
		}


#if NETCFDESIGNTIME
		[
		Category("Data"),
		Description("Indicates the minutes amount to increment/decrement on each button click."),
		DefaultValue(1)
		]
#endif
		public int MinuteIncrement {
			get {
				return (this._minuteIncrement);
			}
			set {
				this._minuteIncrement = value;
			}
		}

#if NETCFDESIGNTIME
		[
		Category("Data"),
		Description("todo"),
		DefaultValue(true)
		]
#endif
		public bool Rotate {
			get {
				return (this._rotate);
			}
			set {
				this._rotate = value;
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.Width = 77;
			this.Height = 20;

			this._SpinButtons = new ReflectionIT.Windows.Forms.SpinButton();
			// 
			// _SpinButtons
			// 
			this._SpinButtons.Left = 40;
			this._SpinButtons.Top = 0;
			this._SpinButtons.SpinUp += new System.EventHandler(this.spinUp_Click);
			this._SpinButtons.SpinDown += new System.EventHandler(this.spinDown_Click);

			this.Controls.Add(_SpinButtons);
		}
		#endregion

		private void spinUp_Click(object sender, System.EventArgs e) {
			this.Focus();
			ReflectionIT.Time t = _time;

			if (_hourSelected) {
				t = t.AddHours(HourIncrement);
			} else {
				if (Rotate) { 
					t = t.AddMinutes(MinuteIncrement);
				} else {
					if (t.Minutes + MinuteIncrement > 59) {
						t = t.AddMinutes(MinuteIncrement - 60);
					} else {
						t = t.AddMinutes(MinuteIncrement);
					}
				}
			}
			while (t.Hours >= 24) {
				t = t.AddHours(-24);
			}

			// Set the time
			if (t > Maximum) {
				this.Time = Maximum;
			} else {
				if (t < Minimum) {
					this.Time = Minimum;
				} else {
					this.Time = t;
				}
			}
		}

		private void spinDown_Click(object sender, System.EventArgs e) {
			this.Focus();
			ReflectionIT.Time t = _time;

			if (_hourSelected) {
				t = t.AddHours(24 - HourIncrement);
			} else {
				if (Rotate) { 
					t = t.AddMinutes((24 * 60) - MinuteIncrement); 
				} else {
					if (t.Minutes - MinuteIncrement < 0) {
						t = t.AddMinutes(60 - MinuteIncrement);
					} else {
						t = t.AddMinutes(MinuteIncrement * -1);
					}
				}
			}
			while (t.Hours >= 24) {
				t = t.AddHours(-24);
			}

			// Set the time
			if (t > Maximum) {
				this.Time = Maximum;
			} else {
				if (t < Minimum) {
					this.Time = Minimum;
				} else {
					this.Time = t;
				}
			}

		}

		#if NETCFDESIGNTIME
		public new Size Size {        // make readonly
			get {
				return base.Size;
			}
			set {
				base.Size = value;
				this.Height = 20;
			}
		}

		//    The control will always keep its defaultsize, so the designer
		//    will not insert a 'new Size' line.
		protected override Size DefaultSize {
			get {
				return new Size(67, 20);
			}
		}

		[
		Browsable(false), 
		EditorBrowsable(EditorBrowsableState.Never),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
		]
		public override string Text {
			get { return base.Text; }
			set { 
				// ignore 
			}
		}
		#endif

		protected override void OnPaint(PaintEventArgs e) {

			base.OnPaint(e);

			string s = System.Globalization.CultureInfo.CurrentUICulture.DateTimeFormat.TimeSeparator;
			int w = Convert.ToInt32(e.Graphics.MeasureString(s, this.Font).Width);

			// Border
			using (Pen pen = new Pen(SystemColors.WindowFrame)) {
				e.Graphics.DrawRectangle(pen,
					this.ClientRectangle.X,
					this.ClientRectangle.Y,
					this.ClientRectangle.Width - 1,
					this.ClientRectangle.Height -1);
			}

			Brush brushH = new SolidBrush(SystemColors.ControlText);
			Brush brushM = new SolidBrush(SystemColors.ControlText);

			e.Graphics.DrawString(s, this.Font, brushH, 18, 2);

			if (this.Focused) {
				if (_hourSelected) { 
					Brush brush = new SolidBrush(SystemColors.Highlight);
					e.Graphics.FillRectangle(brush,
						this.ClientRectangle.X + 3,
						this.ClientRectangle.Y + 3,
						15,
						this.ClientRectangle.Height - 6);
					brush.Dispose();

					brushH = new SolidBrush(SystemColors.HighlightText);
				}
				if (_minutesSelected) { 
					Brush brush = new SolidBrush(SystemColors.Highlight);
					e.Graphics.FillRectangle(brush,
						this.ClientRectangle.X + 3 + 16 + w,
						this.ClientRectangle.Y + 3,
						15,
						this.ClientRectangle.Height - 6);
					brush.Dispose();

					brushM = new SolidBrush(SystemColors.HighlightText);
				}
			}

			e.Graphics.DrawString(string.Format("{0:00}", _time.Hours) , this.Font, brushH, 4, 2);
			e.Graphics.DrawString(string.Format("{0:00}", _time.Minutes) , this.Font, brushM, 20 + w, 2);
			
			brushH.Dispose();
			brushM.Dispose();
		}

		protected override void OnMouseDown(MouseEventArgs e) {
			this.Focus();
			base.OnMouseDown(e);
			_hourSelected = e.X < 18;
			_minutesSelected = e.X > 18 & e.X < 36;
			if (_hourSelected | _minutesSelected) {
				Invalidate();
			}
		}

		protected virtual void OnTimeChanged() {
			if (TimeChanged != null) {
				TimeChanged(this, new EventArgs());
			}
		}

		protected override void OnLostFocus(EventArgs e) {
			base.OnLostFocus(e);
			Invalidate();
		}

		protected override void OnResize(EventArgs e) {
			base.OnResize(e);
			if (_SpinButtons != null) {
				_SpinButtons.Left = this.Width - _SpinButtons.Width;
				Invalidate();
			}
		}

	}
}
