using System;
using System.Drawing;
using System.Windows.Forms;

namespace ReflectionIT.Windows.Forms
{
	internal class UpDownButton : Control {

		public bool Down = true;
		private bool _mouseDown = false;
		private Timer _timer;

		public UpDownButton() {
			this.Width = 14;
			this.Height = 20;
		}

		protected override void OnPaint(PaintEventArgs e) {

			Brush brush = new SolidBrush(Color.Black);

			if (_mouseDown) {
				e.Graphics.FillRectangle(brush, 
					this.ClientRectangle.X,
					this.ClientRectangle.Y,
					this.ClientRectangle.Width,
					this.ClientRectangle.Height);
				brush.Dispose();
				brush = new SolidBrush(Color.White);
			} else {
				using (Pen pen = new Pen(Color.Black)) {
					e.Graphics.DrawRectangle(pen,
						this.ClientRectangle.X,
						this.ClientRectangle.Y,
						this.ClientRectangle.Width - 1,
						this.ClientRectangle.Height -1);
				}
			}

			int marginT = 7;
			int marginB =  this.ClientRectangle.Height - 8;
			int marginL = 3;
			const int marginR = 3;

			if (Down) {
				marginT = this.ClientRectangle.Height - 8;
				marginB = 8;
				marginL = 4;
			}

			Point[] points = new Point[3];
			points[0] = new Point((this.ClientRectangle.Width - this.ClientRectangle.X) / 2, marginT);
			points[1] = new Point(this.ClientRectangle.X + marginL, marginB);
			points[2] = new Point(this.ClientRectangle.Width - marginR, marginB);

			e.Graphics.FillPolygon(brush, points);

			brush.Dispose();
		}

		protected override void OnMouseDown(MouseEventArgs e) {
			base.OnMouseDown(e);
			_mouseDown = true;

			this.Refresh();

			_timer = new Timer();
			_timer.Tick += new EventHandler(TimerTick);
			_timer.Interval = 400;
			_timer.Enabled = true;
		}

		protected override void OnMouseUp(MouseEventArgs e) {
			base.OnMouseUp(e);
			_timer.Enabled = false;
			_timer.Dispose();
			_timer = null;
			_mouseDown = false;
			this.Refresh();
		}

		protected void TimerTick(object sender, EventArgs e) {
			OnClick(new EventArgs());
			_timer.Interval = 200;
		}

	}
}
